
/**
 * error.p -
 *
 * By Alon Blich
 */

function getErrorMsg returns char private

    ( pcError as char,
      pcParam as char ) forward.



define temp-table ttErrorMsg no-undo

    field cError    as char
    field cErrorMsg as char

    index cError is primary unique
          cError.

define stream stIO.

define var cError           as char no-undo.
define var cErrorMsg        as char no-undo.

define var cReturn          as char no-undo.
define var lReturn          as log no-undo.
define var lReturnError     as log no-undo.
define var lReturnNoApply   as log no-undo.



on "close" of this-procedure do:

    delete procedure this-procedure.

end. /* on close */

run initializeProc.

procedure initializeProc private:

    run clearAll.

    run loadErrorMsgFile.

end procedure. /* initializeProc */



procedure err_try:

    run clearAll.

end procedure. /* err_try */

procedure clearAll private:

    assign
        cReturn         = ""
        lReturn         = no
        lReturnError    = no
        lReturnNoApply  = no

        cError          = ""
        cErrorMsg       = ""

            no-error. /* clears the error-status object */

end procedure. /* clearAll */



procedure err_NoError:

    if not error-status:num-messages > 0 then
        return.

    run fetchProErrors.

    stop.

end procedure. /* err_NoError */

procedure err_NoErrorFlag:

    if not error-status:error then
        return.

    run fetchProErrors.

    stop.

end procedure. /* err_NoErrorFlag */

procedure fetchProErrors private:

    define var i as int no-undo.
    define var j as int no-undo.

    assign
        cError      = ""
        cErrorMsg   = ""

    j = error-status:num-messages. do i = 1 to j:

        assign
            cError      = cError
                + ( if cError <> "" then "," else "" )
                + string( error-status:get-number(i) )

            cErrorMsg   = cErrorMsg
                + ( if cErrorMsg <> "" then chr(1) else "" )
                + error-status:get-message(i).

    end. /* 1 to j */

    assign no-error. /* clears the error-status object */

end procedure. /* fetchProErrors */



procedure err_OSError:

    define var i as int no-undo.

    i = os-error.

    if not i = 0 then do:

        assign
            cError      = "OS" + string(i)
            cErrorMsg   = "OS Error (" + string(i) + ")".

        stop.

    end. /* i = 0 */

end procedure. /* err_OSError */



function err_catch returns log

    ( output pcError    as char,
      output pcErrorMsg as char ):

    if cError <> "" then do:

        assign
            pcError     = cError
            pcErrorMsg  = cErrorMsg

            cError      = ""
            cErrorMsg   = "".

        return yes.

    end. /* cError <> "" */

    else do:

        assign
            pcError     = ?
            pcErrorMsg  = ?.

        return no.

    end. /* else */

end function. /* err_catch */

function err_throw returns log

    ( pcError as char,
      pcParam as char ):

    assign
        cError      = pcError
        cErrorMsg   = getErrorMsg( pcError, pcParam ).

    stop.

end function. /* err_throw */

function err_throwExact returns log 

    ( pcError       as char,
      pcErrorMsg    as char ):

    assign
        cError      = pcError
        cErrorMsg   = pcErrorMsg.

    stop.

end function. /* err_throwExact */



function getErrorMsg returns char private

    ( pcError as char,
      pcParam as char ):

    define var str  as char no-undo.
    define var i    as int no-undo.
    define var j    as int no-undo.

    find first ttErrorMsg
         where ttErrorMsg.cError = pcError
         no-error.

    if avail ttErrorMsg then do:

        str = ttErrorMsg.cErrorMsg.

        if  pcParam <> ""
        and pcParam <> ? then do:

            j = min( num-entries( pcParam, chr(1) ), 9 ). do i = 1 to j:

                str = replace( str, "&" + string(i), entry( i, pcParam, chr(1) ) ).

            end.

        end. /* pcParam <> "" */

        str = str + ". ".

    end. /* avail */

    else
    str = "Unknown error. ".

    str = str + "(" + pcError + ")". /* matches progress error message formats */

    return str.

end function. /* getErrorMsg */

procedure loadErrorMsgFile private:

    define var cFileName    as char no-undo.
    define var str1         as char no-undo.
    define var str2         as char no-undo.

    cFileName = search( "errormsg" ).
    if cFileName = ? then return.

    input stream stIO from value( cFileName ).

    repeat:

        assign
            str1 = ""
            str2 = "".

        import stream stIO
            str1
            str2.

        if  not trim( str1 ) begins "#"
        and str1 <> "" and str2 <> ""

        and not can-find(
            first ttErrorMsg
            where ttErrorMsg.cError = str1 ) then do:

            create ttErrorMsg.
            assign
                ttErrorMsg.cError       = str1
                ttErrorMsg.cErrorMsg    = str2.

        end. /* str1 <> "" */

    end. /* repeat */

    input stream stIO close.

end procedure. /* loadErrorMsgFile */



function err_catchReturn returns log

    ( output pcReturn as char ):

    if lReturn then do:

        assign
            pcReturn    = cReturn

            cReturn     = ""
            lReturn     = no.

        return yes.

    end. /* lReturn */

    else do:

        pcReturn = ?.

        return no.

    end. /* else */

end function. /* err_catchReturn */

function err_catchReturnError returns log:

    if lReturnError then do:

        lReturnError = no.

        return yes.

    end. /* lReturnError */

    else return no.

end function. /* err_catchReturnError */

function err_catchReturnNoApply returns log:

    if lReturnNoApply then do:

        lReturnNoApply = no.

        return yes.

    end. /* lReturnNoApply */

    else return no.

end function. /* err_catchReturnNoApply */



function err_return returns log

    ( pcReturn as char ):

    assign
        cReturn = pcReturn
        lReturn = yes.

    stop.

end function. /* err_return */

function err_returnDyn returns char

    ( pcReturn as char ):

    return pcReturn.

end function. /* err_returnDyn */

function err_returnError returns log:

    lReturnError = yes.

    stop.

end function. /* err_returnError */

function err_returnNoApply returns log:

    lReturnNoApply = yes.

    stop.

end function. /* err_returnNoApply */
